<?php
if (! defined('ZENNO_ABSTRACT')) {
    throw new Exception('Connection error in ' . __FILE__);
}

class Document extends ZENNO_ABSTRACT
{

    /**
     * @var int
     */
    protected $_zp_id;

    /**
     * @var bool
     */
    protected $_errorDetected = false;

    /**
     * @var string
     */
    protected $_address;

    /**
     * @var CommandTransfer
     */
    protected $_transfer;
    
    /**
     * 
     * @return Document
     * @param int|string $zp_id
     * @param string $address
     * @param CommandTransfer $transfer
     */
    public function __construct($zp_id, $address, CommandTransfer $transfer)
    {
		$this->_zp_id = ($zp_id == '') ? -1 : $zp_id;
        $this->_address = $address;
        $this->_transfer = $transfer;
    }
    
    /**
     * пустой (не имеет представления на сервере)
     * 
     * @return bool
     */
    protected function _getIsVoid()
    {
        return $this->_zp_id == -1;
    }
    
    
    /**
     * была ли ошибка при выполнении последней команды 
     * 
     * @return bool
     */
    protected function _getErrorDetected()
    {
        return $this->_errorDetected;
    }
    
    /**
     * Id, представляющий вкладку на сервере 
     * 
     * @return int 
     */
    protected function _getZP_ID()
    {
        return $this->_zp_id;
    }
    
    
    /**
     * Родительская вкладка 
     * 
     * @return Tab
     */
    protected function _getParentTab()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_ParentTab', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToTab($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return Tab::GetNull();
        }
    }
    
    
    /**
     * Тело документа
     * 
     * @return HtmlElement 
     */
    protected function _getBody()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_Body', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Голова документа 
     * 
     * @return HtmlElement
     */
    protected function _getHead()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_Head', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Первая форма
     * 
     * @return HtmlElement
     */
    protected function _getMainForm()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_MainForm', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Все формы 
     * 
     * @return HtmlElementCollection 
     */
    protected function _getForms()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_Forms', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    
    /**
     * Первый дочерний элемент 
     * 
     * @return HtmlElement
     */
    protected function _getFirstChild()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_FirstChild', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Url на котором находится документ 
     * 
     * @return String
     */
    protected function _getURL()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_URL', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }


    /**
     * Domain на котором находится документ 
     * 
     * @return string
     */
    protected function _getDomain()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_Domain', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    
    /**
     * Высота документа
     * 
     * @return int
     */
    protected function _getHeight() 
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_Height', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToInt($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return -1;
        }
    }
    
    
    /**
     * Ширина документа 
     * 
     * @return int
     */
    protected function _getWidth() 
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_Width', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToInt($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return -1;
        }
    }
    
    
    /**
     *  Существует ли этот документ
     *  
     * @return bool
     */
    protected function _getIsNull()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_IsNull', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToBoolean($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return true;
        }
    }
    
    
    /**
     * Смещение левого верхнего угла относительно окна таба 
     * 
     * @return object  - point object {x, y}
     */
    protected function _getDisplacementInTabWindow()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_DisplacementInTabWindow', 'zp_id', (string)$this->_zp_id);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToPoint($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return (object)array('x' => 0,
                                 'y' => 0);
        }
    }
    
    
    /// <summary> Выдает элемент, находящийся в указанных координатах </summary>
    /**
     * 
     * @return HtmlElement
     * @param int $x
     * @param int $y
     */
    public function GetElementFromPoint($x, $y)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_GetElementFromPoint',
                array('zp_id', (string)$this->_zp_id,
                      'x', (string)$x,
                      'y', (string)$y));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элемент, по содержащемуся в нем атрибуту
     * 
     * @return HtmlElement
     * @param string $tags
     * @param string $attrName
     * @param string $attrValue
     * @param string $searchKind
     * @param int $number
     */
    public function FindElementByAttribute($tags, $attrName, $attrValue, $searchKind, $number)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_FindElementByAttribute',
                array('zp_id', (string)$this->_zp_id,
                      'tags', $tags,
                      'attrName', $attrName,
                      'attrValue', $attrValue,
                      'searchKind', $searchKind,
                      'number', (string)$number), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элемент, по содержащемуся в нем атрибуту
     * 
     * @return HtmlElementCollection
     * @param string $tags
     * @param string $attrName
     * @param string $attrValue
     * @param string $searchKind
     */
    public function FindElementsByAttribute($tags, $attrName, $attrValue, $searchKind)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_FindElementsByAttribute',
                array('zp_id', (string)$this->_zp_id,
                      'tags', $tags,
                      'attrName', $attrName,
                      'attrValue', $attrValue,
                      'searchKind', $searchKind), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    
    /**
     * Выдает элемент, по id 
     * 
     * @return HtmlElement
     * @param string $id
     */
    public function FindElementById($id)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_FindElementById',
                array('zp_id', (string)$this->_zp_id,
                      'id', $id));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элементы, по id 
     * 
     * @return HtmlElementCollection 
     * @param string $id
     */
    public function FindElementsById($id)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_FindElementsById',
                array('zp_id', (string)$this->_zp_id,
                      'id', $id));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    
    /**
     * Выдает элемент, по name
     * 
     * @return HtmlElement
     * @param string $name
     */
    public function FindElementByName($name)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_FindElementByName',
                array('zp_id', (string)$this->_zp_id,
                      'name', $name));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элементы, по name 
     * 
     * @return HtmlElementCollection
     * @param string $name
     */
    public function FindElementsByName($name)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_FindElementsByName',
                array('zp_id', (string)$this->_zp_id,
                      'name', $name));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    
    /**
     * Выдает элемент, по тэгу 
     * 
     * @return HtmlElement
     * @param string $tag
     * @param int $number
     */
    public function FindElementByTag($tag, $number)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_FindElementByTag',
                array('zp_id', (string)$this->_zp_id,
                      'tag', $tag,
                      'number', (string)$number));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Выдает элементы, по тэгам (поддерживает несколько тэгов)
     * 
     * @return HtmlElementCollection
     * @param string $tags
     */
    public function FindElementsByTags($tags)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_FindElementsByTags',
                array('zp_id', (string)$this->_zp_id,
                      'tags', $tags));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    
    /**
     * Выдать детей (если рекурсивно, то всех детей, а если нет, то первый слой)
     * 
     * @return HtmlElementCollection
     * @param bool $recursive
     */
    public function GetChildren($recursive)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_GetChildren',
                array('zp_id', (string)$this->_zp_id,
                      'recursive', (strtolower($recursive) ? 'true' : 'false')));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHECollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElementCollection::GetNull();
        }
    }
    
    
    /**
     * Выдать фреймы
     * 
     * @return DocumentCollection
     * @param bool $recursive
     */
    public function GetFrames($recursive)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_GetFrames',
                array('zp_id', (string)$this->_zp_id,
                      'recursive', (strtolower($recursive) ? 'true' : 'false')));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToDocumentCollection($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return DocumentCollection::GetNull();
        }
    }
    
    
    /**
     * Выдать текст документа, который видит пользователь
     * 
     * @return String
     * @param bool $recursive
     */
    public function GetPageText($recursive)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_GetPageText',
                array('zp_id', (string)$this->_zp_id,
                      'recursive', (strtolower($recursive) ? 'true' : 'false')));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    
    /**
     * Выдать dom текст документа 
     * 
     * @return string
     * @param bool $recursive
     */
    public function GetDomText($recursive)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_GetPageText',
                array('zp_id', (string)$this->_zp_id,
                      'recursive', (strtolower($recursive) ? 'true' : 'false')));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    
    /**
     * Выполнить скрипт 
     * 
     * @return string
     * @param string $functionName
     * @param array $parameters - array of strings
     */
    public function ExecScript($functionName, $parameters)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_ExecScript',
                array('zp_id', (string)$this->_zp_id,
                      'functionName', $functionName),
                      $parameters);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    
    /**
     * Вставить скрипт 
     * 
     * @return HtmlElement 
     * @param string $scriptText
     */
    public function InsertScript($scriptText)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_InsertScript',
                array('zp_id', (string)$this->_zp_id,
                      'scriptText', $scriptText), array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Создать элемент (сначала будет null т.к. не принадлежит документу)
     * 
     * @return HtmlElement
     * @param string $outerHtml
     */
    public function CreateElement($outerHtml)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_CreateElement',
                array('zp_id', (string)$this->_zp_id,
                      'outerHtml', $outerHtml),
                      null);
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToHE($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return HtmlElement::GetNull();
        }
    }
    
    
    /**
     * Вставить элемент в документ
     * 
     * @return void
     * @param HtmlElement $element
     */
    public function InsertElement($element)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_InsertElement',
                array('zp_id', (string)$this->_zp_id,
                      'element', (string)$element->ZP_ID));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Удалить элемент из документа (станет null т.к. не будет больше принадлежать никакому документу)
     * 
     * @return void
     * @param HtmlElement $element
     */
    public function RemoveElement($element)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_RemoveElement',
                array('zp_id', (string)$this->_zp_id,
                      'element', (string)$element->ZP_ID));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Заменить старый элемент на новый (старый станет null т.к. не будет больше принадлежать никакому документу)
     * 
     * @return void
     * @param HtmlElement $newElement
     * @param HtmlElement $oldElement
     */
    public function ReplaceElement($newElement, $oldElement)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            /**
             * @var string
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                'Doc_ReplaceElement',
                array('zp_id', (string)$this->_zp_id,
                      'newElement', (string)$newElement->ZP_ID,
                      'oldElement', (string)$oldElement->ZP_ID));
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            ZPConverter::ConvertToString($result,
                $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Возвращает пустой документ 
     * 
     * @return Document 
     */
    public static function GetNull()
    {
        return new Document(-1, '', null);
    }


}

