<?php
if (! defined('ZENNO_ABSTRACT')) {
    throw new Exception('Connection error in ' . __FILE__);
}

class DocumentCollection extends ZENNO_ABSTRACT
{
    /**
     * @var array
     */
    protected $_zp_ids = array();
    
    /**
     * @var bool
     */
    protected $_errorDetected = false;
    
    /**
     * @var string
     */
    protected $_address;
    
    /**
     * @var CommandTransfer
     */
    protected $_transfer;
    
    /**
     * @var
     */
    protected $_separator = ':';
    
    /**
     * 
     * @return DocumentCollection
     * @param array|string $zp_ids
     * @param string $address
     * @param string $transfer
     */
    public function __construct($zp_ids, $address, CommandTransfer $transfer)
    {
        $this->_zp_ids = is_array($zp_ids) ? $zp_ids : explode($this->_separator, $zp_ids);

        $this->_address = $address;
        $this->_transfer = $transfer;
    }

    /**
     * пустой (не имеет представления на сервере)
     * 
     * @return bool
     */
    protected function _getIsVoid()
    {
        return $this->_transfer == null;
    }
    
    
    /**
     * была ли ошибка при выполнении последней команды 
     * 
     * @return bool
     */
    protected function _getErrorDetected()
    {
        return $this->_errorDetected;
    }
    
    
    /**
     * Количество документов в коллекции
     * 
     * @return int 
     */
    protected function _getCount()
    {
        return count($this->_zp_ids);
    }
    
    
    /**
     * Выдать документы коллекции в виде массива
     * 
     * @return array - array of Document
     */
    protected function _getDocuments()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $documents = array();
            foreach ($this->_zp_ids as $id) {
                $documents[] = new Document($id, $this->_address, $this->_transfer);
            }
            return $documents;
        } else {
            $this->_errorDetected = true;
            return array();
        }
    }
    
    
    /**
     * выдает документ по номеру в коллекции 
     * 
     * @return Document
     * @param int $number
     */
    public function GetByNumber($number)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid && count($this->_zp_ids) > $number) {
            return new Document($this->_zp_ids[$number], $this->_address, $this->_transfer);
        } else {
            $this->_errorDetected = true;
        }
        return Document::GetNull();
    }
    
    
    /**
     * Вставить документ в коллекцию (в конец) 
     * 
     * @return void
     * @param Document $document
     */
    public function Add($document)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid && $document != null) {
            $this->_zp_ids[] = $document->ZP_ID;
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Добавить коллекцию в коллекцию (в конец)
     * 
     * @return void
     * @param DocumentCollection $documents
     */
    public function AddRange($documents)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid && $documents != null) {
            if (is_a($documents, 'DocumentCollection')) {
                for ($i = 0; $i < count($documents); $i++) {
                    $this->_zp_ids[] = $documents->GetByNumber($i)->ZP_ID;
                }
            } else {
                foreach ($documents as $document) {
                    $this->_zp_ids[] = $document->ZP_ID;
                }
            }
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Вставить документ в коллекцию (с указанного номера)
     * 
     * @return void
     * @param Document $document
     * @param int $index
     */
    public function Insert($document, $index)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid && $document != null) {
            if (count($this->_zp_ids) > $index) {
                $this->_zp_ids[$index] = $document->ZP_ID;
            } else {
                $this->_zp_ids[] = $document->ZP_ID;
            }
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Добавить коллекцию в коллекцию (с указанного номера)
     * 
     * @return void
     * @param DocumentCollection $documents
     * @param int $index
     */
    public function InsertRange($documents, $index)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid && $documents != null) {
            $ids = array();
            if (is_a($documents, 'DocumentCollection')) {
                $ids = $documents->_zp_ids;
            } else {
                foreach ($documents as $document) {
                    $ids[] = $document->ZP_ID;
                }
            }
            if (count($this->_zp_ids) > $index) {
                foreach ($ids as $i => $id) {
                    $this->_zp_ids[$index + $i] = $id;
                }
            } else {
                foreach ($ids as $id) {
                    $this->_zp_ids[] = $id;
                }
            }
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Удаляет документ с указанным номером
     * 
     * @return void
     * @param int $index
     */
    public function Remove($index)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid && isset($this->_zp_ids[$index])) {
            unset($this->_zp_ids[$index]);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Выдает номер документа в коллекции или -1 если документа нет в коллекции
     * 
     * @return int
     * @param Document $document
     */
    public function IndexOf($document)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            foreach ($this->_zp_ids as $i => $id) {
                if ($id == $document->ZP_ID) {
                    return $i;
                }
            }
        } else {
            $this->_errorDetected = true;
        }
        return -1;
    }
    
    
    /**
     * Возвращает пустой элемент 
     * 
     * @return DocumentCollection 
     */
    public static function GetNull()
    {
        return new DocumentCollection(array(), '', null);
    }
}