<?php
if (! defined('ZENNO_ABSTRACT')) {
    throw new Exception('Connection error in ' . __FILE__);
}

class HtmlElementCollection extends ZENNO_ABSTRACT
{
    /**
     * @var array array of int
     */
    protected $_zp_ids = array();

    /**
     * @var bool
     */
    protected $_errorDetected = false;

    /**
     * @var string
     */
    protected $_address;

    /**
     * @var CommandTransfer
     */
    protected $_transfer = null;

    /**
     * @var string
     */
    protected $_separator = ':';

    /**
     * 
     * @return HtmlElementCollection
     * @param array|string $zp_ids
     * @param string $address
     * @param string $transfer
     */
    public function __construct($zp_ids, $address, CommandTransfer $transfer)
    {
        $this->_zp_ids = is_array($zp_ids) ? $zp_ids : explode($this->_separator, $zp_ids);

        $this->_address = $address;
        $this->_transfer = $transfer;
    }
    
    
    /**
     * пустой (не имеет представления на сервере)
     * 
     * @return bool
     */
    protected function _getIsVoid()
    {
        return $this->_transfer == null;
    }
    
    
    /**
     * была ли ошибка при выполнении последней команды 
     * 
     * @return bool
     */
    protected function _getErrorDetected()
    {
        return $this->_errorDetected;
    }
    
    
    /**
     * Количество элементов в коллекции 
     * 
     * @return int 
     */
    protected function _getCount()
    {
        return count($this->_zp_ids);
    }
    
    
    /**
     * Выдать элементы коллекции в виде массива 
     * 
     * @return array - array of HtmlElement
     */
    protected function _getElements()
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $elements = array();
            foreach ($this->_zp_ids as $id) {
                $elements[] = new HtmlElement($id, $this->_address, $this->_transfer);
            }
            return $elements;
        } else {
            $this->_errorDetected = true;
            return array();
        }
    }
    
    
    /**
     * выдает элемент по номеру в коллекции 
     * @return HtmlElement
     * @param int $number
     */
    public function GetByNumber($number)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid && count($this->_zp_ids) > $number) {
            return new HtmlElement($this->_zp_ids[$number], $this->_address, $this->_transfer);
        } else {
            $this->_errorDetected = true;
        }
        return HtmlElement::GetNull();
    }
    
    
    /**
     * Вставить элемент в коллекцию (в конец)
     * 
     * @return void
     * @param HtmlElement $element
     */
    public function Add(HtmlElement $element)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid && $element != null) {
            $this->_zp_ids[] = $element->ZP_ID;
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Добавить коллекцию в коллекцию (в конец)
     * 
     * @return void
     * @param HtmlElementCollection $elements
     */
    public function AddRange(HtmlElementCollection $elements)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid && $elements != null) {
            if (is_a($elements, 'HtmlElementCollection')) {
                for ($i = 0; $i < count($elements); $i++) {
                    $this->_zp_ids[] = $elements->GetByNumber($i)->ZP_ID;
                } 
            } else {
                foreach ($elements as $element) {
                    $this->_zp_ids[] = $element->ZP_ID;
                }
            }

        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Вставить элемент в коллекцию (с указанного номера)
     * 
     * @return void
     * @param HtmlElement $element
     * @param int $index
     */
    public function Insert(HtmlElement $element, $index)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid && $element != null) {
            if (count($this->_zp_ids) > $index) {
                $this->_zp_ids[$index] = $element->ZP_ID;
            } else {
                $this->_zp_ids[] = $element->ZP_ID;
            }
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Добавить коллекцию в коллекцию (с указанного номера)
     * 
     * @return void
     * @param HtmlElementCollection|array  $elements - HtmlElementCollection object or array of HtmlElement
     * @param int $index
     */
    public function InsertRange(HtmlElementCollection $elements, $index)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid && $elements != null) {
            $ids = array();
            if (is_a($elements, 'HtmlElementCollection')) {
                $ids = $elements->_zp_ids;
            } else {
                foreach ($elements as $element) {
                    $ids[] = $element->ZP_ID;
                }
            }
            if (count($this->_zp_ids) > $index) {
                foreach ($ids as $i => $id) {
                    $this->_zp_ids[$index + $i] = $id;
                }
            } else {
                foreach ($ids as $id) {
                    $this->_zp_ids[] = $id;
                }
            }
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Удаляет элемент с указанным номером 
     * 
     * @return void
     * @param int $index
     */
    public function Remove($index)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid && isset($this->_zp_ids[$index])) {
            unset($this->_zp_ids[$index]);
        } else {
            $this->_errorDetected = true;
        }
    }
    
    
    /**
     * Выдает номер элемента в коллекции или -1 если элемента нет в коллекции
     * 
     * @return int
     * @param HtmlElement $element
     */
    public function IndexOf(HtmlElement $element)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            foreach ($this->_zp_ids as $i => $id) {
                if ($id == $element->ZP_ID) {
                    return $i;
                }
            }
        } else {
            $this->_errorDetected = true;
        }
        return -1;
    }
    
    
    /**
     * Со всех элементов коллекции берутся атрибуты с указанным именем и собираются в текст каждый с начала строки
     * или
     * Со всех элементов коллекции берутся атрибуты с указанным именем и собираются в текст через указанный разделитель
     * 
     * @return string
     * @param string $attrName
     * @param null|string
     */
    public function AttributesToString($attrName, $delemiter = null)
    {
        $this->_errorDetected = false;
        if (! $this->isVoid) {
            $parameters = array('ids', $this->IdsToString(),
                                'attrName', $attrName);
            if ($delemiter != null) {
                $parameters[] = 'delemiter';
                $parameters[] = $delemiter;
            }
            /**
             * @var stridng
             */
            $message = CommandTransfer::CreateMessage($this->_address,
                ($delemiter == null ? 'EC_AttributesToString' : ''),
                $parameters, array());
            /**
             * @var string
             */
            $result = $this->_transfer->SendMessageWait($message);
            return ZPConverter::ConvertToString($result,
                $this->_address, $this->_transfer, &$this->_errorDetected);
        } else {
            $this->_errorDetected = true;
            return '';
        }
    }
    
    
    /**
     * 
     * @return string
     */
    protected function IdsToString()
    {
        $idsStr = array();
        foreach ($this->_zp_ids as $id) {
            $idsStr[] = $id;
        }
        return explode(';', $idsStr);
    }
    
    
    /**
     * Возвращает пустой элемент
     * 
     * @return HtmlElementCollection 
     */
    public static function GetNull()
    {
        return new HtmlElementCollection(array(), '', null);
    }
}